from datetime import date
import uuid
from django.db import models
from django.forms import ModelForm
from django.core.validators import validate_unicode_slug, FileExtensionValidator


class Category(models.Model):
    id = models.UUIDField(primary_key=True, default=uuid.uuid4, editable=False)
    name = models.CharField(max_length=64, validators=[validate_unicode_slug], default="New Category")
    full_name = models.CharField(max_length=512, editable=False, default="")
    parent = models.ForeignKey("self", on_delete=models.SET_NULL, blank=True, null=True)

    def __str__(self):
        if self.parent is None or self.parent == self:
            return self.name
        return f"{self.parent}>{self.name}"

    def save(self, *args, **kwargs):
        self.full_name = str(self)
        super().save(*args, **kwargs)
        for child in self.__class__.objects.filter(parent=self):
            child.save()

    @property
    def tree(self):
        if self.parent is None or self.parent == self:
            return [self.name]
        return self.parent.tree + [self.name]

    class Meta:
        ordering = ["full_name"]


class CategoryForm(ModelForm):
    class Meta:
        model = Category
        fields = ["name", "parent"]


class Transaction(models.Model):
    id = models.UUIDField(primary_key=True, default=uuid.uuid4, editable=False)
    name = models.CharField(max_length=256, default="New Transaction")
    description = models.TextField(null=True, blank=True)
    value = models.DecimalField(max_digits=12, decimal_places=2, default=0)
    date = models.DateField(default=date.today)
    trader = models.CharField(max_length=128, blank=True, null=True)
    category = models.ForeignKey(
        Category, on_delete=models.SET_NULL, blank=True, null=True
    )

    def __str__(self):
        res = f"{self.date} {self.name}: {self.value}€"
        if self.category:
            return f"{res} ({self.category})"
        return res

    class Meta:
        ordering = ["-date"]


class TransactionForm(ModelForm):
    class Meta:
        model = Transaction
        fields = ["name", "description", "value", "date", "category"]


class Invoice(models.Model):
    id = models.UUIDField(primary_key=True, default=uuid.uuid4, editable=False)
    name = models.CharField(max_length=256, default="New Invoice")
    file = models.FileField(
        upload_to="invoices/", validators=[FileExtensionValidator(["pdf"])]
    )
    transaction = models.ForeignKey(Transaction, on_delete=models.CASCADE)

    def __str__(self):
        return f"{self.name}: {self.transaction}"

    def delete(self, *args, **kwargs):
        self.file.delete()
        super().delete(*args, **kwargs)


class InvoiceForm(ModelForm):
    class Meta:
        model = Invoice
        fields = ["name", "file"]
